/*
 ** Module   : VT100SwingRenderer.java
 ** Abstract : SWING VT-100 renderer implementation.
 **
 ** Copyright (c) 2014, Golden Code Development Corporation.
 ** ALL RIGHTS RESERVED. Use is subject to license terms.
 **
 **           Golden Code Development Corporation
 **                      CONFIDENTIAL
 **
 ** -#- -I- --Date-- ----------------Description-----------------
 ** 001 MAG 20140318 First version.
 ** 002 MAG 20140507 Fix cursor bug for web clients.
 */

package com.goldencode.p2j.ui.client.chui.driver.swing;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.chui.driver.web.*;
import com.goldencode.terminal.*;

/**
 * Implements VT-100 renderer for Swing and AJAX ChUI clients.
 *  
 * @author mag
 *
 */
public class VT100SwingRenderer 
extends VT100AbstractRenderer<Cell>
{      
   /** ChUI simulator */
   private ChuiSimulator sim = null;
   
   /** True if ChuiSimulator is for a web client */
   private boolean isWeb = false;
   
   /**
    * Constructor.
    * 
    * @param   sim
    *          Swing renderer.
    */
   public VT100SwingRenderer(ChuiSimulator sim)   
   {
      // ChUI renderer
      this.sim = sim;
      
      // Check if it's a web client
      isWeb = sim instanceof ChuiWebSimulator;
           
      // Video memory data
      data = sim.getScreenData();
      cols = sim.getColumns();
      rows = sim.getRows();

      // Scroll window
      begScroll = 0;
      endScroll = rows - 1;
   }
   
   /**
    * Play a message beep.
    */
   @Override
   public void beep()
   {
      if (isWeb)
      {
         ((ChuiWebSimulator) sim).beep();
      }
      else
      {
         java.awt.Toolkit.getDefaultToolkit().beep();
      }
   }

   /**
    * Set cursor visibility.
    * 
    * @param   showCursor
    * 
    */
   @Override
   public void setCursorVisible(boolean showCursor)
   {
      sim.setCursorStatus(showCursor);
      updateScreen();
   }
   
   /**
    * Get attribute mask.
    * 
    * @param   id
    *          Attribute id.
    * <p>
    *          0 - NORMAL 
    *          1 - BLINK 
    *          2 - BOLD 
    *          3 - UNDERLINE 
    *          4 - REVERSE
    *          
    * @return  attribute mask.
    */
   @Override
   public int attributeMask(int id)
   {
      switch (id)
      {
         case 0:
            return ColorAttribute.NORMAL.getValue();
         case 1:
            return ColorAttribute.BLINK.getValue();
         case 2:
            return ColorAttribute.BOLD.getValue();
         case 3:
            return ColorAttribute.UNDERLINE.getValue();
         case 4:
            return ColorAttribute.REVERSE.getValue();
         default:
            return ColorAttribute.NORMAL.getValue();
      }      
   }   

   /**
    * Update screen.
    */
   @Override
   protected void updateScreen()
   {
      // repaint
      sim.triggerRepaint(true);
      sim.setCursor(col, row);
   }
   
   /**
    * Create an empty cell.
    * 
    * @return  An empty video memory cell.
    */
   @Override
   protected Cell emptyCell()
   {
      return ChuiSimulator.EMPTY_CELL;
   }

   /**
    * Build a video memory cell.
    * 
    * @param   text
    *          Cell character.
    * @param   attribute
    *          Color attributes
    * @param   color
    *          Color pair FG/BG
    * 
    * @return  A new video memory cell.
    */   
   @Override
   protected Cell makeCell(char text, int attribute, int color)
   {
      // Box characters are not properly rendered in SWING having BOLD attribute. 
      if ((text >= '\u2500') && (text <= '\u25ff'))
      {
         attribute &= 0xFD;
      }
      
      // return a new Cell instance
      return new Cell(text, new com.goldencode.p2j.ui.Color(attribute, color));
   }
}
